create or replace package body GEOGRAFIC_PKG is

  -- Comprova l'existncia d'un estat.
  --
  -- %param P_CODEST  Estat a comprovar
  --
  -- %return Indicador sobre si existeix un estat amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION ESTAT_EXISTEIX (P_CODEST    ESTAT.CODEST%TYPE
                          ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from ESTAT est
                      where est.CODEST = P_CODEST);

    return (vExisteix > 0);
  END;

  -- Comprova si l'estat indicat t dades asociades
  -- Dades a comprovar:
  --  Regions
  --  Ciutadans
  --  Censos
  --
  -- %param P_CODEST  Estat a comprovar
  --
  -- %return Indicador sobre si l'estat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION ESTAT_TE_DADES_ASOCIADES (P_CODEST    ESTAT.CODEST%TYPE
                                    ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (-- Regions
                    Select 1
                      from REGIO reg
                      where reg.EST_CODEST = P_CODEST) or
            exists (-- Ciutadans
                    Select 1
                      from CIUTADA ctd
                      where ctd.EST_CODEST = P_CODEST) or
            exists (-- Censos
                    Select 1
                      from CENS cen
                      where cen.EST_CODEST = P_CODEST);

    return (vExisteix > 0);
  END;

  -- Dona d'alta un nou estat.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar el nom de lestat
  -- 102            Lestat ja existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_NOMEST  Nom de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_ALTA (P_CODEST        ESTAT.CODEST%TYPE,
                        P_NOMEST        ESTAT.NOMEST%TYPE,
                        P_RSP       OUT VARCHAR2
                       ) IS

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rEst       ESTAT%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('ESTAT_ALTA', 'Alta d''un estat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMEST        : ' || P_NOMEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_NOMEST is null then
      P_RSP := '101-El nom de l''estat s obligatori';
    elsif ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '102-Ja existeix un estat amb el CODI ' || P_CODEST;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rEst.CODEST := P_CODEST;
      rEst.NOMEST := P_NOMEST;

      Insert into ESTAT
        values rEst;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades d'un estat (el nom)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar el nom de lestat
  -- 102            Lestat no existeix
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_NOMEST  Nou nom de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_MODIFICACIO (P_CODEST        ESTAT.CODEST%TYPE,
                               P_NOMEST        ESTAT.NOMEST%TYPE,
                               P_RSP       OUT VARCHAR2
                              ) IS

    Cursor cEst is
      Select est.*
        from ESTAT est
        where est.CODEST = P_CODEST
        for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rEst       ESTAT%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('ESTAT_MODIFICACIO', 'Modifica dades d''un estat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMEST        : ' || P_NOMEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_NOMEST is null then
      P_RSP := '101-El nom de l''estat s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '102-No existeix cap estat amb el CODI ' || P_CODEST;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      -- Obtenim bloqueig sobre el registre de l'estat a actualitzar
      -- i actualitzem el nom
      Open cEst;
      Fetch cEst into rEst;

      rEst.NOMEST := P_NOMEST;

      Update ESTAT est
        set row = rEst
        Where current of cEst;

      Close cEst;

    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa un estat. Noms es permet donar de baixa un estat si aquest
  -- t dades associades (regions, censos...)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Lestat no existeix
  -- 102            Lestat t dades associades. No es permet donar-lo de baixa
  -- 900            Error inesperat
  --
  --
  -- %param P_CODEST  Codi de lestat
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE ESTAT_BAIXA (P_CODEST        ESTAT.CODEST%TYPE,
                         P_RSP       OUT VARCHAR2
                        ) IS

    Cursor cEst is
      Select est.*
        from ESTAT est
        where est.CODEST = P_CODEST
        for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rEst       ESTAT%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('ESTAT_MODIFICACIO', 'Modifica dades d''un estat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '101-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif ESTAT_TE_DADES_ASOCIADES (P_CODEST) then
      P_RSP := '102-L''estat t dades associades. No es permet donar-lo de baixa';
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      -- Obtenim bloqueig sobre el registre de l'estat a elimina
      -- i l'eliminem
      Open cEst;
      Fetch cEst into rEst;
      Close cEst;

      Delete from ESTAT est
        where est.CODEST = P_CODEST;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Comprova l'existncia d'una regi.
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  --
  -- %return Indicador sobre si existeix una regi amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION REGIO_EXISTEIX (P_CODEST    REGIO.EST_CODEST%TYPE,
                           P_CODREG    REGIO.CODREG%TYPE
                          ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from REGIO reg
                      where reg.EST_CODEST = P_CODEST and
                            reg.CODREG = P_CODREG);

    return (vExisteix > 0);
  END;

  -- Comprova si la regi indicada t dades asociades
  -- Dades a comprovar:
  --  Localitats
  --  Ciutadans
  --  Censos
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  --
  -- %return Indicador sobre si la regi t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION REGIO_TE_DADES_ASOCIADES (P_CODEST    REGIO.EST_CODEST%TYPE,
                                     P_CODREG    REGIO.CODREG%TYPE
                                    ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (-- Localitats
                    Select 1
                      from LOCALITAT loc
                      where loc.EST_CODEST = P_CODEST and
                            loc.REG_CODREG = P_CODREG) or
            exists (-- Ciutadans
                    Select 1
                      from CIUTADA ctd
                      where ctd.EST_CODEST = P_CODEST and
                            ctd.REG_CODREG = P_CODREG) or
            exists (-- Censos
                    Select 1
                      from CENS cen
                      where cen.EST_CODEST = P_CODEST and
                            cen.REG_CODREG = P_CODREG);

    return (vExisteix > 0);
  END;

  -- Dona dalta una regi.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar el nom de la regi
  -- 103            Lestat no existeix
  -- 104            La regi ja existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_ALTA (P_CODEST        REGIO.EST_CODEST%TYPE,
                        P_CODREG        REGIO.CODREG%TYPE,
                        P_NOMREG        REGIO.NOMREG%TYPE,
                        P_RSP       OUT VARCHAR2
                       ) IS

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rReg       REGIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('REGIO_ALTA', 'Dona d''alta una regi');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMREG        : ' || P_NOMREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif P_NOMREG is null then
      P_RSP := '102-El NOM de la regi s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '103-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '104-Ja existeix una regi amb el CODI ' || P_CODREG;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rReg.EST_CODEST := P_CODEST;
      rReg.CODREG     := P_CODREG;
      rReg.NOMREG     := P_NOMREG;

      Insert into REGIO
        values rReg;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades duna regio (nom).
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar el nom de la regi
  -- 103            Lestat no existeix
  -- 104            La regi no existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_MODIFICACIO (P_CODEST        REGIO.EST_CODEST%TYPE,
                               P_CODREG        REGIO.CODREG%TYPE,
                               P_NOMREG        REGIO.NOMREG%TYPE,
                               P_RSP       OUT VARCHAR2
                              ) IS

    Cursor cReg is
      Select reg.*
        from REGIO reg
        where reg.EST_CODEST = P_CODEST and
              reg.CODREG = P_CODREG
          for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rReg       REGIO%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('REGIO_MODIFICACIO', 'Modifica dades d''una regi');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMREG        : ' || P_NOMREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif P_NOMREG is null then
      P_RSP := '102-El NOM de la regi s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '103-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '104-No existeix cap regi amb el CODI ' || P_CODREG;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      -- Obtenim bloqueig sobre el registre de la regi a actualitzar
      -- i actualitzem el nom
      Open cReg;
      Fetch cReg into rReg;

      rReg.NOMREG := P_NOMREG;

      Update REGIO reg
        set row = rReg
        Where current of cReg;

      Close cReg;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa una regi. Noms es permet donar de baixa una regi si
  -- aquesta no t dades associades (localitats, censos...)
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Lestat no existeix
  -- 103            La regi no existeix
  -- 104            La regi t dades associades i no es permet la seva baixa.
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE REGIO_BAIXA (P_CODEST        REGIO.EST_CODEST%TYPE,
                         P_CODREG        REGIO.CODREG%TYPE,
                         P_RSP       OUT VARCHAR2
                        ) IS

    Cursor cReg is
      Select 1
        from REGIO reg
        where reg.EST_CODEST = P_CODEST and
              reg.CODREG = P_CODREG
          for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('REGIO_BAIXA', 'Elimina una regi');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '102-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '103-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif REGIO_TE_DADES_ASOCIADES (P_CODEST, P_CODREG) then
      P_RSP := '104-La regi t dades associades. No es permet donar-la de baixa';
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      -- Obtenim bloqueig sobre el registre de la regi a actualitzar
      -- i actualitzem el nom
      Open cReg;
      Close cReg;

      Delete from REGIO reg
        where reg.EST_CODEST = P_CODEST and
              reg.CODREG = P_CODREG;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Comprova l'existncia d'una localitat.
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  -- %param P_CODLOC  Localitat a comprovar
  --
  -- %return Indicador sobre si existeix una localitat amb el codi indicat
  --           {*} TRUE  Exiteix
  --           {*} FALSE No existeix
  FUNCTION LOCALITAT_EXISTEIX (P_CODEST    LOCALITAT.EST_CODEST%TYPE,
                               P_CODREG    LOCALITAT.REG_CODREG%TYPE,
                               P_CODLOC    LOCALITAT.CODLOC%TYPE
                              ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (Select 1
                      from LOCALITAT loc
                      where loc.EST_CODEST = P_CODEST and
                            loc.REG_CODREG = P_CODREG and
                            loc.CODLOC = P_CODLOC);

    return (vExisteix > 0);
  END;

  -- Comprova si la localitat indicada t dades asociades
  -- Dades a comprovar:
  --  Ciutadans
  --  Censos
  --
  -- %param P_CODEST  Estat a comprovar
  -- %param P_CODREG  Regi a comprovar
  -- %param P_CODLOC  Localitat a comprovar
  --
  -- %return Indicador sobre si la localitat t dades asociades
  --           {*} TRUE  T dades asociades
  --           {*} FALSE No t dades asociades
  FUNCTION LOCALITAT_TE_DADES_ASOCIADES (P_CODEST    LOCALITAT.EST_CODEST%TYPE,
                                         P_CODREG    LOCALITAT.REG_CODREG%TYPE,
                                         P_CODLOC    LOCALITAT.CODLOC%TYPE
                                        ) RETURN BOOLEAN IS

    vExisteix    number;
  BEGIN
    Select count(1)
      into vExisteix
      from dual
      where exists (-- Ciutadans
                    Select 1
                      from CIUTADA ctd
                      where ctd.EST_CODEST = P_CODEST and
                            ctd.REG_CODREG = P_CODREG and
                            ctd.LOC_CODLOC = P_CODLOC) or
            exists (-- Censos
                    Select 1
                      from CENS cen
                      where cen.EST_CODEST = P_CODEST and
                            cen.REG_CODREG = P_CODREG and
                            cen.LOC_CODLOC = P_CODLOC);

    return (vExisteix > 0);
  END;

  -- Dona dalta una localitat.
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Sha dinformar el nom de la regi
  -- 104            Lestat no existeix
  -- 105            La regi no existeix
  -- 106            La localitat ja existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_ALTA (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                            P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                            P_CODLOC        LOCALITAT.CODLOC%TYPE,
                            P_NOMLOC        LOCALITAT.NOMLOC%TYPE,
                            P_RSP       OUT VARCHAR2
                           ) IS

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rLoc       LOCALITAT%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('LOCALITAT_ALTA', 'Dona d''alta una localitat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMLOC        : ' || P_NOMLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif P_CODLOC is null then
      P_RSP := '102-El CODI de la localitat s obligatori';
    elsif P_NOMLOC is null then
      P_RSP := '103-El NOM de la localitat s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '104-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '105-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '106-Ja existeix una localitat amb el CODI ' || P_CODLOC;
    end if;

    -- s tot correcte, realitzem l'alta a la taula
    If P_RSP = 'OK' then

      rLoc.EST_CODEST := P_CODEST;
      rLoc.REG_CODREG := P_CODREG;
      rLoc.CODLOC     := P_CODLOC;
      rLoc.NOMLOC     := P_NOMLOC;

      Insert into LOCALITAT
        values rLoc;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Modifica les dades duna localitat (nom).
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Sha dinformar el nom de la regi
  -- 104            Lestat no existeix
  -- 105            La regi no existeix
  -- 106            La localitat no existeix
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_MODIFICACIO (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                                   P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                                   P_CODLOC        LOCALITAT.CODLOC%TYPE,
                                   P_NOMLOC        LOCALITAT.NOMLOC%TYPE,
                                   P_RSP       OUT VARCHAR2
                                  ) IS

    Cursor cLoc is
      Select loc.*
        from LOCALITAT loc
        where loc.EST_CODEST = P_CODEST and
              loc.REG_CODREG = P_CODREG and
              loc.CODLOC = P_CODLOC
          for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
    rLoc       LOCALITAT%ROWTYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('LOCALITAT_MODIFICACIO', 'Modifica dades d''una localitat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_NOMLOC        : ' || P_NOMLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif P_CODLOC is null then
      P_RSP := '102-El CODI de la localitat s obligatori';
    elsif P_NOMLOC is null then
      P_RSP := '103-El NOM de la localitat s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '104-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '105-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif not LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '106-No existeix cap localitat amb el CODI ' || P_CODLOC;
    end if;

    -- s tot correcte, realitzem la modificaci
    If P_RSP = 'OK' then

      Open cLoc;
      Fetch cLoc into rLoc;

      rLoc.NOMLOC := P_NOMLOC;

      Update LOCALITAT loc
        set row = rLoc
        where current of cLoc;

      Close cLoc;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;

  -- Dona de baixa una localitat. Noms es pot donar de baixa si no t
  -- dades associades
  --
  -- Taula d'errors del procs
  --
  -- Codi derror   Descripci
  -- ------------   ---------------------------------
  -- 100            Sha dinformar lestat
  -- 101            Sha dinformar la regi
  -- 102            Sha dinformar la localitat
  -- 103            Lestat no existeix
  -- 104            La regi no existeix
  -- 105            La localitat no existeix
  -- 106            La localitat t dades asociades. No es permet la seva baixa
  -- 900            Error inesperat
  --
  -- %param P_CODEST  Codi de lestat al que pertany la regi
  -- %param P_CODREG  Codi de la regi
  -- %param P_NOMREG  Nom de la regi
  -- %param P_RSP     Codi de resposta. Si el procs dalta acabat amb xit,
  --                  retorna el valor OK, sin retorna un codi derror seguit
  --                  de la descripci, seguint el format CODI-DESCRIPCIӔ
  PROCEDURE LOCALITAT_BAIXA (P_CODEST        LOCALITAT.EST_CODEST%TYPE,
                             P_CODREG        LOCALITAT.REG_CODREG%TYPE,
                             P_CODLOC        LOCALITAT.CODLOC%TYPE,
                             P_RSP       OUT VARCHAR2
                            ) IS

    Cursor cLoc is
      Select 1
        from LOCALITAT loc
        where loc.EST_CODEST = P_CODEST and
              loc.REG_CODREG = P_CODREG and
              loc.CODLOC = P_CODLOC
          for update nowait;

    vIdeLog    LOG_INFO.IDELOG%TYPE;
  BEGIN
    -- Informaci de log
    vIdeLog := LOG_PKG.INICIAR_LOG ('LOCALITAT_BAIXA', 'Dona de baixa una localitat');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Parmetres d''entrada');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, '---------------------');
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODEST        : ' || P_CODEST);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODREG        : ' || P_CODREG);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'P_CODLOC        : ' || P_CODLOC);
    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, ' ');

    P_RSP := 'OK';

    -- Comprovacions prvies
    If P_CODEST is null then
      P_RSP := '100-El CODI de l''estat s obligatori';
    elsif P_CODREG is null then
      P_RSP := '101-El CODI de la regi s obligatori';
    elsif P_CODLOC is null then
      P_RSP := '102-El CODI de la localitat s obligatori';
    elsif not ESTAT_EXISTEIX (P_CODEST) then
      P_RSP := '103-No existeix cap estat amb el CODI ' || P_CODEST;
    elsif not REGIO_EXISTEIX (P_CODEST, P_CODREG) then
      P_RSP := '104-No existeix cap regi amb el CODI ' || P_CODREG;
    elsif not LOCALITAT_EXISTEIX (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '105-No existeix cap localitat amb el CODI ' || P_CODLOC;
    elsif LOCALITAT_TE_DADES_ASOCIADES (P_CODEST, P_CODREG, P_CODLOC) then
      P_RSP := '106-La localitat t dades asociades. No es permet la seva baixa.';
    end if;

    -- s tot correcte, realitzem la modificaci
    If P_RSP = 'OK' then

      Open cLoc;
      Close cLoc;

      Delete from LOCALITAT loc
        where loc.EST_CODEST = P_CODEST and
              loc.REG_CODREG = P_CODREG and
              loc.CODLOC = P_CODLOC;
    end if;

    LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
    LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_FINALITZAT);
  EXCEPTION
    -- Erros no esperats
    When others then
      P_RSP := '900-' || SQLERRM(SQLCODE);
      LOG_PKG.REGISTRAR_DETALL_LOG (vIdeLog, 'Resultat del procs: ' || P_RSP);
      LOG_PKG.FINALITZAR_LOG (vIdeLog, LOG_PKG.cESTAT_AMB_ERRORS);
  END;
end;
/

exit
/

